      PROGRAM TESTPROCMAP
*
*     -- BLACS example code --
*     University of Tennessee, Knoxville
*
*     Written by Clint Whaley 7/26/94               (rwhaley@cs.utk.edu)
*     Slightly modified by Antoine Petitet 6/5/95   (petitet@cs.utk.edu)
*
*     This program shows how to map processes to a grid using
*     BLACS_GRIDMAP.
*
*     .. Parameters ..
      INTEGER            NIN, NOUT
      PARAMETER          ( NIN = 5, NOUT = 6 )
*     ..
*     .. Local Scalars ..
      INTEGER            IAM, ICALLER, ICONTXT, NPCOL, NPROCS, NPROW,
     $                   MYCOL, MYROW
*     ..
*     .. Local Arrays ..
      INTEGER            IWORK( 1000 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GET, BLACS_GRIDINFO, BLACS_GRIDINIT,
     $                   BLACS_PINFO, BLACS_SETUP, PROCMAP
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          INT, SQRT
*     ..
*     .. External Functions ..
      INTEGER            BLACS_PNUM
      EXTERNAL           BLACS_PNUM
*
*     Determine my process number and the number of processes in
*     machine
*
      CALL BLACS_PINFO( IAM, NPROCS )
*
*     If underlying system needs additional setup, do it now
*
      IF( NPROCS.LT.1 ) THEN
         IF( IAM.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9999 )
            READ( NIN, FMT = * ) NPROCS
         END IF
         CALL BLACS_SETUP( IAM, NPROCS )
      END IF
*
*     Set up process grid that is as close to square as possible
*
      NPROW = INT( SQRT( REAL( NPROCS ) ) )
      NPCOL = NPROCS / NPROW
*
*     Get default system context, and define grid
*
      CALL BLACS_GET( 0, 0, ICONTXT )
      CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     If I'm not in grid, go to end of program
*
      IF( MYROW.EQ.-1 )
     $  GO TO 10
*
*     Get my process ID from my grid coordinates
*
      ICALLER = BLACS_PNUM( ICONTXT, MYROW, MYCOL )
*
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      IF( IAM.EQ.0 ) THEN
         WRITE( NOUT, FMT = * )
         WRITE( NOUT, FMT = * )
      END IF
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      WRITE( NOUT, FMT = 9998 ) 'Row-major   : ', ICALLER, MYROW, MYCOL
*
*     Go to column-major ordering ...
*
      CALL PROCMAP( ICONTXT, 2, 0, NPROW, NPCOL, IWORK )
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      IF( IAM.EQ.0 ) THEN
         WRITE( NOUT, FMT = * )
         WRITE( NOUT, FMT = * )
      END IF
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      WRITE( NOUT, FMT = 9998 ) 'Column-major: ', ICALLER, MYROW, MYCOL
*
*     Back to row-major ordering ...
*
      CALL PROCMAP( ICONTXT, 1, 0, NPROW, NPCOL, IWORK )
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      IF( IAM.EQ.0 ) THEN
         WRITE( NOUT, FMT = * )
         WRITE( NOUT, FMT = * )
      END IF
      CALL BLACS_BARRIER( ICONTXT, 'All' )
      WRITE( NOUT, FMT = 9998 ) 'Row-major   : ', ICALLER, MYROW, MYCOL
*
   10 CONTINUE
*   
      CALL BLACS_EXIT( 0 )
*
 9999 FORMAT( 1X, 'How many processes in machine ?' )
 9998 FORMAT( 1X, A14, 'Process #', I6, ' has coordinates (',
     $        I6, ',', I6, ').' )
*
      STOP
*
      END
*
      SUBROUTINE PROCMAP( CONTEXT, MAPPING, BEGPROC, NPROW, NPCOL,
     $                    IMAP )
*
*     -- BLACS example code --
*     Written by Clint Whaley 7/26/94.
*     ..
*     .. Scalar Arguments ..
      INTEGER            CONTEXT, MAPPING, BEGPROC, NPROW, NPCOL
*     ..
*     .. Array Arguments ..
      INTEGER            IMAP( NPROW, * )
*     ..
*
*  Purpose
*  =======
*
*  PROCMAP maps NPROW*NPCOL processes starting from process BEGPROC to
*  the grid in a variety of ways depending on the parameter MAPPING.
*
*  Arguments
*  =========
*
*  CONTEXT      (output) INTEGER
*               This integer is used by the BLACS to indicate a context.
*               A context is a universe where messages exist and do not
*               interact with other context's messages.  The context
*               includes the definition of a grid, and each process's
*               coordinates in it.
*
*  MAPPING      (input) INTEGER
*               Way to map processes to grid.  Choices are:
*               1 : row-major natural ordering
*               2 : column-major natural ordering
*
*  BEGPROC      (input) INTEGER
*               The process number (between 0 and NPROCS-1) to use as
*               {0,0}.  From this process, processes will be assigned
*               to the grid as indicated by MAPPING.
*
*  NPROW        (input) INTEGER
*               The number of process rows the created grid 
*               should have.
*
*  NPCOL        (input) INTEGER
*               The number of process columns the created grid
*               should have.
*
*  IMAP         (workspace) INTEGER array of dimension (NPROW, NPCOL)
*               Workspace where the array which maps the 
*               processes to the grid will be stored for the
*               call to GRIDMAP.
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER            I, J, K, NPROCS, TMPCONTXT
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_PINFO, BLACS_GRIDINIT, BLACS_GRIDMAP
*     ..
*     .. External Functions ..
      INTEGER            BLACS_PNUM
      EXTERNAL           BLACS_PNUM
*     ..
*     .. Executable Statements ..
*
*     See how many processes there are in the system
*
      CALL BLACS_PINFO( I, NPROCS )
      IF( NPROCS-BEGPROC .LT. NPROW*NPCOL ) THEN
         WRITE(*,*) 'Not enough processes for grid'
         STOP
      END IF
*
*     Temporarily map all processes into 1 x NPROCS grid
*
      CALL BLACS_GET( 0, 0, TMPCONTXT )
      CALL BLACS_GRIDINIT( TMPCONTXT, 'Row-major', 1, NPROCS )
      K = BEGPROC
*
*     If we want a row-major natural ordering
*
      IF( MAPPING.EQ.1 ) THEN
         DO 20 I = 1, NPROW
            DO 10 J = 1, NPCOL
               IMAP( I, J ) = BLACS_PNUM( TMPCONTXT, 0, K )
               K = K + 1
   10       CONTINUE
   20    CONTINUE
*
*     If we want a column-major natural ordering
*
      ELSE IF( MAPPING.EQ.2 ) THEN
         DO 40 J = 1, NPCOL
            DO 30 I = 1, NPROW
               IMAP( I, J ) = BLACS_PNUM( TMPCONTXT, 0, K )
               K = K + 1
   30       CONTINUE
   40    CONTINUE
      ELSE
         WRITE(*,*) 'Uknown mapping.'
         STOP
      END IF
*
*     Get the system context TMPCONTXT is based on
*
      CALL BLACS_GET( TMPCONTXT, 10, CONTEXT )
*
*     Apply the new mapping to form desired context
*
      CALL BLACS_GRIDMAP( CONTEXT, IMAP, NPROW, NPROW, NPCOL )
*
*     Free temporary context
*
      CALL BLACS_GRIDEXIT( TMPCONTXT )
*
      RETURN
*
      END
